% LinealVeff.m
% 15-Dec-92 18:35:52 

% Ajuste de rectas completas y = a + b x, a datos (xi,yi)  
% empleando el metodo de varianza efectiva. 

% Junto con este archivo se requieren estos tres adicionales:

% 1) datos.m      -> Contiene los datos y sus PESOS
% 2) errorbar2.m  -> Subrutina que permite graficar datos con incertezas
%                    en dos variables. 
% 3) f_linveff.m  -> Subrutina donde se evalua S(a,b)


% *** INICIO ***

% Generalmente conviene borrar todos los resultados de corridas previas y  
% los graficos que hayan quedado.  
clear all
close all hidden

% Como vamos a tener que usar ciertas variables en una subrutina externa, 
% por comodidad las declaramos GLOBALES
global xi yi sxi syi

% INGRESO DE DATOS 
% La siguiente instruccion lee el archivo "datos.m"

datos                         % datos de Pearson con pesos de York
                              % (xi,yi) datos a ser ajustados
                              % (sxi,syi) dispersiones asignadas por York 

% GRAFICO DE LOS DATOS CON ERRORES

figure(1)

% Se usa una subrutina de graficacion especial que permite graficar 
% los puntos con sus errores

errorbar2(xi',sxi',yi',syi','.')

% Se puede agregar un titulo y leyendas a los ejes

title('Datos de Pearson con pesos de York')
xlabel('x')
ylabel('y = \Psi^{2} (3 x^{4})')   % Se pueden usar algunos comandos LaTeX

% Agregamos un texto aclaratorio en el grafico
text (0,2,'Grafico de los datos. Oprimir enter para continuar'); pause 

% Y despues lo borramos, para que no se superponga con los siguientes
% textos. 
text (0,2,'Grafico de los datos. Oprimir enter para continuar','Color','w')

hold on                     % Guardamos el grafico para despues. 


% AJUSTE APROXIMADO

% Se realiza un ajuste aproximado (sin pesos) con el proposito de obtener 
% un buen punto de partida para el ajuste mediante varianza efectiva. 

% Ajuste aproximado mediante un polinomio de grado 1. 

ajusteaprox = polyfit(xi,yi,1);     % Ajuste de los puntos (xi, yi) 
                                    % mediante un polinomio de grado 1. 
                                    % La funcion POLYFIT devuelve los 
                                    % coefs del polinomio en orden
                                    % decreciente de potencias.

yiaprox     = polyval(ajusteaprox,xi);  % yi ajustados en forma aproximada

aaprox      = ajusteaprox(2);           % ordenada al origen aproximada
baprox      = ajusteaprox(1);           % pendiente aproximada


% Grafico anterior superpuesto (gracias a la instruccion 'hold on') 
% con el resultado del ajuste aproximado (se grafican los puntos 
% mediante una '*' para cada uno, una linea llena entre ellos '-', 
% y se pide usar color rojo 'r')

plot(xi,yiaprox,'-*r')

text (0,2,'Ajuste aproximado. Oprimir enter para continuar'); pause
text (0,2,'Ajuste aproximado. Oprimir enter para continuar','Color','w'); 

hold on                 % Queremos volver a usar este grafico tambien



% AJUSTE MEDIANTE VARIANZA EFECTIVA

% Minimizacion de S(a,b). 

% Aqui se emplea la subrutima "fminsearch" de Matlab. 

options = optimset('TolX',1e-6,'TolFun',1e-6);  % Definimos algunas opciones 

[pajustado, count] = fminsearch('f_linveff',[aaprox baprox]'); 
  
% Ya termino la minimizacion. Extraigo los valores de a y b
a   = pajustado(1);
b   = pajustado(2);

% Calculo de los puntos ajustados Xi e Yi

di      = yi - a - b*xi;                      % distancias verticales
Wi      = (wxi.*wyi)./(wxi + b^2*wyi);        % pesos efectivos
si      = Wi.*di.^2;
Sred    = sum(si);                            % S reducida
lamdai  = -Wi.*di;                            % mult. Lagrange
Xi      = xi - lamdai*b./wxi;                 % (Xi, Yi) 
Yi      = yi + lamdai./wyi;

% Grafico de los puntos ajustados (la opcion '-og' significa: 
% unir los puntos mediante segmentos, graficar un circulo en cada 
% punto, y usar color verde)

figure(1)
hold on
plot(xi,yiaprox,'-*r')
hold on
plot(Xi,Yi,'-og')


% Calculo de incertezas 

Fi     = xi + 2*b*(di.*Wi./wxi);
A      = -sum(Wi);
B      = -sum(Wi.*Fi);
C      = sum((((di.*Wi).^2)./wxi) - Wi.*Fi.^2);
Delta  = A*C - B^2; 

dadx   = Wi.*(b*C + B.*(di - b*Fi))/Delta;
dady   = Wi.*(B*Fi - C)/Delta;
dbdx   = -Wi.*(b*B + A*(di - b*Fi))/Delta;
dbdy   = Wi.*(B - A*Fi)/Delta;

sa     = sqrt( sum( (dadx .* sxi).^2 + (dady .* syi).^2) );
sb     = sqrt( sum( (dbdx .* sxi).^2 + (dbdy .* syi).^2) );


texto(1) = {'Ajuste mediante Varianza Efectiva'};
texto(2) = {''};
texto(3) = {['Ordenada  = ' num2str(a) ' \pm ' num2str(sa)]};
texto(4) = {['Pendiente = ' num2str(b) ' \pm ' num2str(sb)]};
text(0,2,texto,'HorizontalAlignment','left')


hold off % Liberamos el grafico y terminamos, por ahora

return

% Inicio de variaciones (pruebenlo si les interesa)
clf

text(0,0,'Inicio de variaciones','sc'); pause
p = [];
xi0 = xi; yi0 = yi;

for jvarss = 1:1000

xi = xi0 + sxi.*randn(size(xi0));

yi = yi0 + syi.*randn(size(yi0));

pj = fminsearch('f_linveff',[aaprox baprox]'); 

p = [p; pj']
end

[mean(p); std(p)] 

% Grabacion de resultados MC sobre estimacion de dispersiones en

% save terr1 Sab 28 Nov 92
%      terr2 Dom 29           En m: al 0.4% ; c: al 0.2%